@@ -83,6 +83,7 @@ class MeshConductor:
         mesh_obj=None,
         process=False,
         fix_normals=True,
+        basis_name="inner",
         resistivity=1.68 * 1e-8,
         thickness=1e-4,
         **kwargs
@@ -105,6 +106,8 @@ class MeshConductor:
             If True,  Trimesh will pre-process the mesh.
         fix_normals: boolean
             If True,  normals+winding should be set so that they always point "out" from the origin.
+        basis_name: string
+            Which basis to use, must be 'inner', 'vertex' or 'suh'. See class docstring
         Resistivity: float or array (Nfaces)
             Resistivity value in Ohm/meter
         Thickness: float or array (Nfaces)
@@ -112,14 +115,54 @@ class MeshConductor:
         kwargs:
             Additional options with default settings are:
                 'outer_boundaries':None,
-                'mass_lumped':False,
-                'resistance_full_rank': True,
+                'resistance_full_rank': False,
                 'inductance_nchunks':None,
                 'basis_name':'inner' (other: suh, vertex)
                 'N_suh': 100
+                'sph_normalization': 'default'
+                'sph_radius': 1
                 'N_sph': 5
                 'approx_far': True
                 'approx_far_margin': 2
+             
+            
+        Notes
+        ------
+        
+        **outer_boundaries**
+        int or array_like, indices of outer boundaries given by utils.find_boundaries(). 
+        One boundary index per mesh component. If None, outer_boundaries are set to 
+        the longest boundary in each mesh component. When using basis 'inner', the outer boundary
+        vertex values are fixed to zero.
+        
+        **resistance_full_rank** (Boolean)
+        If True, applies inflation to the resistance matrix in order to increase
+        the rank by one. By default, is False. Don't set to True without knowing what you are doing.
+        
+        **inductance_nchunks** (int or None)
+        Number of serial chunks to split self-inductance computation into, saving memory but taking more time.
+        When approx_far is True, using more chunks is more efficient (multiply by 10-100x)
+        If None (default), attempts to set number of chunks automatically based on the amount of free memory.
+        Unfortunately, this estimation is not perfect.
+
+        **N_suh**
+        Number of surface harmonics to use if basis_name is 'suh'
+        
+        **sph_normalization**
+        'default' (Ylm**2 integrated over solid angle to 1) or
+        'energy' (field energy of basis fields normalized to 1 in R-ball)
+        
+        **sph_radius**
+        If sph_normalization is 'energy', defines the radius of the inner expansion
+        
+        **N_sph**
+        Number of spherical harmonics degrees (l-degrees) to use for the spherical harmonics coupling computation
+        
+        **approx_far** (Boolean)
+        If True, usesimple quadrature for points far from the source triangles when computing self-inductance
+        
+        **approx_far_margin** (non-negative float)
+        Cut-off distance for "far" points measured in mean triangle side length.
 
 
         """
@@ -155,11 +198,11 @@ class MeshConductor:
         # Populate options dictionary with defaults if not specified
         self.opts = {
             "outer_boundaries": None,
-            "mass_lumped": False,
             "resistance_full_rank": False,
             "inductance_nchunks": None,
-            "basis_name": "inner",
             "N_suh": 100,
+            "sph_normalization": "default",
+            "sph_radius": 1,
             "N_sph": 5,
             "inductance_quad_degree": 2,
             "approx_far": True,
@@ -207,7 +250,8 @@ class MeshConductor:
         self.vert2inner = utils.vert2inner(self.mesh, self.inner_vertices, self.holes)
 
         # Sets basis for first time, calling self.set_basis()
-        self.set_basis(self.opts["basis_name"])
+        self.basis_name = basis_name
+        self.set_basis(self.basis_name)
 
     def set_basis(self, basis_name):
         """ The data is stored in vertex basis i.e. every
@@ -257,7 +301,7 @@ class MeshConductor:
                     if b[0] in c:
                         b_labels[m] = n
             b_lengths = np.array([len(b) for b in self.boundaries])
-            # Determine outer_boundaries by finding the boundary of max lenght
+            # Determine outer_boundaries by finding the boundary of max length
             # for each component
             outer_boundaries = []
             b_inds = np.arange(len(self.boundaries))
@@ -315,7 +359,7 @@ class MeshConductor:
         Compute and return mesh mass matrix.
 
         """
-        mass = mass_matrix(self.mesh, self.opts["mass_lumped"])
+        mass = mass_matrix(self.mesh, lumped=False)
         return mass
 
     @property
@@ -399,6 +443,33 @@ class MeshConductor:
 
         return M
 
+    def set_sph_options(self, **kwargs):
+        """
+        Set or reset options related to the spherical harmonics (SPH) coupling of the 
+        MeshConductor object. If SPH couplings have already been computed, these will be 
+        flushed.
+    
+
+        Parameters
+        ----------
+        **kwargs 
+            Any parameter related to sph, namely "N_sph", "sph_normalization", "sph_radius" 
+
+        Returns
+        -------
+        None.
+
+        """
+
+        # Flush old results
+        self._alpha_coupling = None
+        self._beta_coupling = None
+
+        # Assign new values to opts, filter out opts not related to sph
+        for key, val in kwargs.items():
+            if key in ("N_sph", "sph_normalization", "sph_radius"):
+                self.opts[key] = val
+
     @property
     def sph_couplings(self):
         """
@@ -415,7 +486,12 @@ class MeshConductor:
         """
         if self._alpha_coupling is None:
             print("Computing coupling matrices")
-            Calpha, Cbeta = compute_sphcoeffs_mesh(self.mesh, self.opts["N_sph"])
+            Calpha, Cbeta = compute_sphcoeffs_mesh(
+                self.mesh,
+                self.opts["N_sph"],
+                self.opts["sph_normalization"],
+                self.opts["sph_radius"],
+            )
             # Store the results for further use
             self._alpha_coupling = Calpha
             self._beta_coupling = Cbeta
@@ -439,6 +515,12 @@ class MeshConductor:
             self.matrices["resistance"] = None  # Flush old matrix
             self._resistance()  # Re-compute with new parameters
 
+        if (name in ("resistivity", "thickness")) and self.matrices[
+            "resistance"
+        ] is not None:
+            self.matrices["resistance"] = None  # Flush old matrix
+            self._resistance()  # Re-compute with new parameters
+
     def plot_mesh(self, cull_front=False, cull_back=False, **kwargs):
         """
         Simply plot the mesh surface in mayavi. kwargs are passed to
@@ -529,7 +611,7 @@ class CouplingMatrix:
         self.points = np.array([])
         self.matrix = np.array([])
 
-    def __call__(self, points, *fun_args, **kwargs):
+    def __call__(self, points, s=None, *fun_args, **kwargs):
         """
         Returns the output of self.function(self.parent, points).
         If some output has already been computed, use pre-computed values instead
@@ -539,6 +621,10 @@ class CouplingMatrix:
         ----------
             points: (N_points, ... ) numpy array
                 Array containing query points
+            s: (Nbasis,) numpy array
+                If None, return coupling matrix, 
+                if not None, return field, i.e, coupling_matrix @ s
+                default None
             *fun_args: additional arguments
                 Optional, additional arguments that are passed to self.function
 
@@ -583,24 +669,35 @@ class CouplingMatrix:
 
             M = self.matrix[m_existing_point_idx]
 
-        if self.matrix.ndim == 2:
-            M = M @ self.parent.basis
+        if s is None:
+            if self.matrix.ndim == 2:
+                M = M @ self.parent.basis
 
-        elif self.matrix.ndim == 3:
+            elif self.matrix.ndim == 3:
+
+                # Handle both sparse and dense basis matrices quickly
+                if issparse(self.parent.basis):
+                    Mnew = []
+                    for n in range(3):
+                        Mnew.append(M[:, n, :] @ self.parent.basis)
+                    M = np.swapaxes(np.array(Mnew), 0, 1)
+                else:
+                    M = np.einsum("ijk,kl->ijl", M, self.parent.basis)
 
-            # Handle both sparse and dense basis matrices quickly
-            if issparse(self.parent.basis):
-                Mnew = []
-                for n in range(3):
-                    Mnew.append(M[:, n, :] @ self.parent.basis)
-                M = np.swapaxes(np.array(Mnew), 0, 1)
             else:
-                M = np.einsum("ijk,kl->ijl", M, self.parent.basis)
+                raise ValueError("Matrix dimensions not ok")
 
+            return M
         else:
-            raise ValueError("Matrix dimensions not ok")
+            if self.matrix.ndim == 2:
+                field = M @ (self.parent.basis @ s)
+            elif self.matrix.ndim == 3:
+                ss = self.parent.basis @ s
+                field = np.einsum("ijk,k->ij", M, ss)
+            else:
+                raise ValueError("Matrix dimensions not ok")
 
-        return M
+            return field
 
 
 class StreamFunction(np.ndarray):
@@ -681,7 +778,7 @@ class StreamFunction(np.ndarray):
         """
         Returns the stream function in inner basis
         """
-        if self.mesh_conductor.basis_name == "inner":
+        if self.basis_name == "inner":
             return self
 
         return self.vert2inner @ self.basis @ self
@@ -691,18 +788,30 @@ class StreamFunction(np.ndarray):
         """
         Stream-function resistive power
         """
-        R = self.mesh_conductor.resistance
-        return self.T @ R @ self
+
+        # Compute resistance matrix if not present
+        if self.mesh_conductor.matrices["resistance"] is None:
+            self.mesh_conductor.resistance
+
+        R = self.mesh_conductor.matrices["resistance"]  # Always in vertex basis
+
+        return self.T @ self.basis.T @ R @ self.basis @ self
 
     @property
     def magnetic_energy(self):
         """
         Stream-function magnetic energy
         """
-        M = self.mesh_conductor.inductance
-        return 0.5 * self.T @ M @ self
 
-    def coil_inductance(Nloops):
+        # Compute inductance matrix if not present
+        if self.mesh_conductor.matrices["inductance"] is None:
+            self.mesh_conductor.inductance
+
+        M = self.mesh_conductor.matrices["inductance"]  # Always in vertex basis
+
+        return 0.5 * self.T @ self.basis.T @ M @ self.basis @ self
+
+    def coil_inductance(self, Nloops):
         """
         
 
@@ -715,8 +824,8 @@ class StreamFunction(np.ndarray):
         inductance: scalar (float)
 
         """
-        scaling = Nloops / (self.max() - self.min())
-        L_approx = 2 * sf.magnetic_energy * (scaling ** 2)
+        scaling = Nloops / (self.vert.max() - self.vert.min())
+        L_approx = 2 * self.magnetic_energy * (scaling ** 2)
 
         return L_approx
 
@@ -765,10 +874,8 @@ class StreamFunction(np.ndarray):
             Vector containing the scalar function value for each contour line
         """
         return LineConductor(
-            scalar_contour(
-                self.mesh_conductor.mesh,
-                self.vert,
-                N_contours=N_contours,
-                contours=contours,
-            )
+            mesh=self.mesh_conductor.mesh,
+            scalars=self.vert,
+            N_contours=N_contours,
+            contours=contours,
         )
